%
%  Subject : Approximation of functions using narrow-base trial functions 
%
%  Example Ej_3_1_2
%
%         Wj = Nl --> Galerkin
%         Nm = piecewise constant @ each element
%         see Zienkiewicz & Morgan page 97 
% 

% Number of elements to be used
Nx = 20;

% grid points
x  = (0:Nx)'/Nx;

% collocation points
xe = 0.5*(x(1:end-1)+x(2:end));

% function to be approximated
phi = ffun_Ej_3_1(xe);

% function used to match end values
psi = 0*(phi(1)+(phi(end)-phi(1))*xe);

% shape functions evaluated at element centroide
% being the piecewise constant orthogonal functions
K = diag(abs(diff(x)));

f = zeros(Nx,1);
for k=1:Nx
    psi = (0:20)'/20; xx=psi*(x(k+1)-x(k))+x(k);
    f(k,1) = trapz(xx,ffun_Ej_3_1(xx));
end

% solver
a = K\f;

% Visualization
figure(1);clf;
plot(x,ffun_Ej_3_1(x),'m',xe,zeros(size(xe)),'r+',x,zeros(size(x)),'bo',xe,a,'k*')
Xh=[]; Yh=[];
for k=1:Nx
    figure(1);hold on
    line([x(k),x(k+1)],a(k)*[1,1])
    Xh = [Xh,[x(k),x(k+1)]];
    Yh = [Yh,a(k)*[1,1]];
end
figure(1);hold on;line(Xh,Yh);
xlabel('X');ylabel('phi');title('1D piecewise constant - Galerkin aproximation ')
