%
%  Subject : Discretization techniques using narrow-base trial functions 
%
%  Example Ej_4_1
%
%           approximation in 1D : heat equation Dirichlet BC
%           example 3.1 , page 111, Zienkiewicz & Morgan
%

% Number of elements to be used
Nx = 3;

% boundary conditions
phi_lef = 0;  % phi(x=0)
phi_rig = 1;  % phi(x=1)

% grid points and mesh
xnod  = (0:Nx)'/Nx;
icone = [(1:Nx)',(2:Nx+1)'];

[numel,nen]=size(icone);

Ke = zeros(numel,2,2);
fe = zeros(numel,2);

% loop over each element
for k=1:numel
    psi = (0:10)'/10; 
    node1 = icone(k,1);
    node2 = icone(k,2);
    xx  = psi*(xnod(node2,1)-xnod(node1,1))+xnod(node1,1);            
    [N,L_N] = shape_function(xx);        
    for mi=1:2
        for mj=1:2
            Ke(k,mi,mj) = Ke(k,mi,mj) + trapz(xx,N(:,mi).*N(:,mj)) + trapz(xx,L_N(:,mi).*L_N(:,mj));    
        end
    end    
end

% gather Ke and fe in Kg and fg
Kg = zeros(Nx+1,Nx+1);
fg = zeros(Nx+1,1);
for k=1:numel
    for mi=1:nen
        for mj=1:nen
            Kg(icone(k,mi),icone(k,mj))=Kg(icone(k,mi),icone(k,mj)) + Ke(k,mi,mj);        
        end
        fg(icone(k,mi),1) = fg(icone(k,mi),1) + fe(k,mi);
    end
end

% reducing the global system for fixations
free = (2:Nx)';
fg(free,1) = fg(free,1) - Kg(free,1)*phi_lef - Kg(free,Nx+1)*phi_rig;
Kg = Kg(free,free);
fg = fg(free,1);
% solver
phi = Kg\fg;

phi = [phi_lef;phi;phi_rig];

% Visualization
figure(1);clf;
x = xnod(:,1);
xe = 0.5*(x(1:end-1)+x(2:end));
% theoretical solution
coef = 1/(exp(1)-1/exp(1));
phi_th = coef*(exp(x)-exp(-x));
plot(x,zeros(size(x)),'bo',x,phi,'k*',x,phi_th,'r-')
xlabel('X');ylabel('phi');title('1D heat equation with piecewise linear Galerkin aproximation - Dirichlet BCs ')
errphi = norm(phi-phi_th);

xnod=[ [xnod , 0*xnod] ; [xnod , max(xnod)*0.1*ones(size(xnod,1),1)] ];
icone = [icone , icone(:,[2,1])+Nx+1];
figure(2);clf;pltmsh(xnod,icone);pfnodes(xnod,icone);axis equal
